// Elektronischer Doppelwrfel
// Gregor Dorfbauer
// http://www.raush.at/
// timer routine by peter danegger

#include <inttypes.h>
#include <avr/io.h>
#ifndef F_CPU
#define F_CPU 4433619UL     /* Quarz mit 4.433619 Mhz  */
#endif
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define u08 uint8_t

//compatibility defines for 2313 (timer routine by peter danegger)
#ifndef OCR1A
#define OCR1A OCR1	// 2313 support
#endif

#ifndef WGM12
#define WGM12 CTC1	// 2313 support
#endif

#define XTAL		F_CPU	// nominal value

#define DEBOUNCE	256L		// debounce clock (256Hz = 4msec)

#define KEYPRESS_LENGTH	25

#define KEY1 PD3
#define KEY2 PD2

#define WUERFEL_1 PD6
#define WUERFEL_0 PB0

#define WUERFEL1 (1<<PB7)
#define WUERFEL2 ((1<<PB6)|(1<<PB1))
#define WUERFEL3 ((1<<PB7)|(1<<PB6)|(1<<PB1))
#define WUERFEL4 ((1<<PB6)|(1<<PB4)|(1<<PB3)|(1<<PB1))
#define WUERFEL5 ((1<<PB6)|(1<<PB4)|(1<<PB3)|(1<<PB1)|(1<<PB7))
#define WUERFEL6 ((1<<PB6)|(1<<PB4)|(1<<PB3)|(1<<PB1)|(1<<PB5)|(1<<PB2))

void OutputDice1(void);
void OutputDice2(void);

u08 key1;	//debounce-counters
u08 key2;

u08 dice1;	//dice values
u08 dice2;

u08 dice1_grow;	//show-register (5 seconds)
u08 dice2_grow;

u08 wuerfel[] = { WUERFEL1, WUERFEL2, WUERFEL3,
			WUERFEL4, WUERFEL5, WUERFEL6 };

u08 random_clk;
u08 prescaler;	//timer-prescaler (1 second)

//timer compare interrupt, every 4 ms
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
#if XTAL % DEBOUNCE
  OCR1A = XTAL / DEBOUNCE - 1;		// compare DEBOUNCE - 1 times
#endif
  if( --prescaler == 0 ){
    prescaler = (u08)DEBOUNCE;
#if XTAL % DEBOUNCE			// handle remainder
    OCR1A = XTAL / DEBOUNCE + XTAL % DEBOUNCE - 1; // compare once per second
#endif
	//one second
	if(dice1_grow > 0)
		dice1_grow--;
	if(dice2_grow > 0)
		dice2_grow--;
  }

  if(random_clk%2)	//mplex
  	OutputDice1();
  else
	OutputDice2();
  
  random_clk++;
	
// key debouncing and input routine
if(!(PIND & (1 << KEY1)))	// key 1 pressed
	key1++;
else
	key1 = 0;

if(!(PIND & (1 << KEY2)))		// key2
	key2++;
else
	key2 = 0;


	if(key1 == KEYPRESS_LENGTH)
	{
		dice1 = (random_clk%6);
		dice1_grow=5;
	}
	if(key2 == KEYPRESS_LENGTH)
	{
		//offset value (3) - if anybody manages to press the
		//buttons exactly at the same time, there are different
		// dices shown.
		dice2 = ((random_clk+3)%6);	
		dice2_grow=5;
	}	
}

void OutputDice1(void)
{
if(dice1_grow != 0) //dice will be shown 5 seconds
{
	PORTD &= ~(1<<WUERFEL_1);
	PORTB &= (1<<WUERFEL_0);
	PORTB |= wuerfel[dice1] | (1<<WUERFEL_0);
}
else
{
	PORTB &= ~(1<<WUERFEL_0);
}
}
void OutputDice2(void)
{
if(dice2_grow != 0) //dice will be shown 5 seconds
{
	PORTD |= (1<<WUERFEL_1);
	PORTB = 0x00;
	PORTB |= wuerfel[dice2];
}
else
{
	PORTD &= ~(1<<WUERFEL_1);
}
}

int main(void)
{	
	DDRB = 0xFF;		// 7 leds + 1 mplex-line
	DDRD = (1<<WUERFEL_1);	//PD6 output (one mplex-line)

	PORTD |= ~(1<<WUERFEL_1); // enable pullup resistors

	// timer 1 config 
	TCCR1B = 1<<WGM12^1<<CS10;	// divide by 1
					// clear on compare
	OCR1A = XTAL / DEBOUNCE - 1;
	TCNT1 = 0;
	
	prescaler = (u08)DEBOUNCE;
	
	TIMSK = 1<<OCIE1A;
	

	key1 = 0;
	key2 = 0;

	dice1 = 0;
	dice2 = 0;

	dice1_grow = 0;		//no dices shown at startup
	dice1_grow = 0;
	
	// enable interrupts
	sei();
	
	// garfield's nap attack, energy saving.
	set_sleep_mode(SLEEP_MODE_IDLE);
	while(1)
	{
		sleep_mode();
	}

	return 0;
}
